@@ -857,7 +857,7 @@ class ImageRefIsARedirect(IronicException):
             redirect_url=redirect_url)
 
 
-class ConcurrentActionLimit(IronicException):
+class ConcurrentActionLimit(TemporaryFailure):
     # NOTE(TheJulia): We explicitly don't report the concurrent
     # action limit configuration value as a security guard since
     # if informed of the limit, an attacker can tailor their attack.
@@ -883,3 +883,7 @@ class FirmwareComponentNotFound(NotFound):
 
 class InvalidNodeInventory(Invalid):
     _msg_fmt = _("Inventory for node %(node)s is invalid: %(reason)s")
+
+
+class InvalidImage(ImageUnacceptable):
+    _msg_fmt = _("The requested image is not valid for use.")
